@@ -1,5 +1,5 @@
 """
-Toolkit to convert ROOT histograms to matplotlib figures.
+Core implementation of the module
 """
 
 ################ Import python libraries
@@ -11,8 +11,6 @@ import ROOT
 import re
 import copy
 
-## imported_pyplot = False
-
 
 ################ Define constants
 
@@ -77,10 +75,22 @@ class Hist2D(object):
         plot = self.col(**kwargs)
         plt.colorbar(plot)
         return plot
+    def box(self, maxsize=40, **kwargs):
+        """Draw a box plot with size indicating content using pyplot.scatter
+
+        The data will be normalized, with the largest box using a marker of
+        size maxsize (in points).
+        """
+        _load_pyplot()
+        x = np.hstack([self.x for i in range(self.nbinsy)])
+        y = np.hstack([[yval for i in range(self.nbinsx)] for yval in self.y])
+        maxvalue = np.max(self.content)
+        sizes = self.content.flatten() / maxvalue * maxsize
+        plot = plt.scatter(x, y, sizes, marker='s', **kwargs)
         
 class Hist(object):
     """A container to hold the parameters from a ROOT histogram."""
-    def __init__(self, hist, label="__nolabel__", title=None,
+    def __init__(self, hist, replace=None, label="__nolabel__", title=None,
                  xlabel=None, ylabel=None):
         try:
             if not hist.InheritsFrom("TH1"):
@@ -102,34 +112,36 @@ class Hist(object):
                              for i in range(n)])
         self.y      = array([hist.GetBinContent(i) for i in range(1, n + 1)])
         self.yerr   = array([hist.GetBinError(  i) for i in range(1, n + 1)])
+        self.width  = array([self.xedges[i+1] - self.xedges[i]
+                             for i in range(self.nbins)])
         self.underflow = hist.GetBinContent(0)
         self.overflow  = hist.GetBinContent(self.nbins + 1)
-        self.title  = hist.GetTitle()
-        self.xlabel = hist.GetXaxis().GetTitle()
-        self.ylabel = hist.GetYaxis().GetTitle()
-        if title : self.title  = title
-        if xlabel: self.xlabel = xlabel
-        if ylabel: self.ylabel = ylabel
-        self.label = label
-    def show_titles(self, replace=None):
+        self.title  = _parse_latex(hist.GetTitle(), replace)
+        self.xlabel = _parse_latex(hist.GetXaxis().GetTitle(), replace)
+        self.ylabel = _parse_latex(hist.GetYaxis().GetTitle(), replace)
+        if title : self.title  = _parse_latex(title, replace)
+        if xlabel: self.xlabel = _parse_latex(xlabel, replace)
+        if ylabel: self.ylabel = _parse_latex(ylabel, replace)
+        self.label = _parse_latex(label, replace)
+    def show_titles(self):
         """Add the titles defined in the ROOT histogram to the figure."""
-        plt.title( _parse_latex(self.title , replace))
-        plt.xlabel(_parse_latex(self.xlabel, replace))
-        plt.ylabel(_parse_latex(self.ylabel, replace))
-    def hist(self, replace=None, **kwargs):
+        plt.title(self.title)
+        plt.xlabel(self.xlabel)
+        plt.ylabel(self.ylabel)
+    def hist(self, **kwargs):
         """
         Generate a matplotlib hist figure.
 
         All additional keyword arguments will be passed to pyplot.hist.
         """
         _load_pyplot()
-        plot = plt.hist(self.x, weights=self.y, bins=self.xedges, **kwargs)
+        plot = plt.hist(self.x, weights=self.y, bins=self.xedges,
+                        label=self.label, **kwargs)
         if self.binlabels is not None:
             plt.xticks(np.arange(self.nbins), self.binlabels)
         plt.xlim(self.xedges[0], self.xedges[-1])
         return plot
-    def errorbar(self, xerr=False, yerr=False,
-                 replace=None, **kwargs):
+    def errorbar(self, xerr=False, yerr=False, **kwargs):
         """
         Generate a matplotlib errorbar figure.
 
@@ -145,18 +157,15 @@ class Hist(object):
             plt.xticks(np.arange(self.nbins), self.binlabels)
         plt.xlim(self.xedges[0], self.xedges[-1])
         return errorbar
-    def bar(self, yerr=False, replace=None, **kwargs):
+    def bar(self, yerr=False, **kwargs):
         """
         Generate a matplotlib bar figure.
 
         All additional keyword arguments will be passed to pyplot.bar.
         """
-        if yerr:
-            kwargs['yerr'] = self.yerr
-        width = np.array([self.xedges[i+1] - self.xedges[i]
-                          for i in range(self.nbins)])
+        if yerr: kwargs['yerr'] = self.yerr
         _load_pyplot()
-        bar = plt.bar(self.xedges[0:-1], self.y, width, label=self.label,
+        bar = plt.bar(self.xedges[0:-1], self.y, self.width, label=self.label,
                       **kwargs)
         return bar
     def TH1F(self, name=""):
@@ -179,7 +188,7 @@ class Hist(object):
         quotient.yerr = np.array([lower_err, upper_err])
         return quotient
     def __div__(self, denominator):
-        return divide_wilson(self, denominator)
+        return self.divide_wilson(denominator)
 
 class HistStack(object):
     """
@@ -188,20 +197,21 @@ class HistStack(object):
     When plotting, the title and the x and y labels of the last Hist added
     will be used unless specified otherwise in the constructor.
     """
-    def __init__(self, hists=None, title=None, xlabel=None, ylabel=None):
+    def __init__(self, hists=None, title=None, xlabel=None, ylabel=None,
+                 replace=None):
         self.hists  = []
         self.kwargs = []
-        self.title  = title
-        self.xlabel = xlabel
-        self.ylabel = ylabel
+        self.title  = _parse_latex(title, replace)
+        self.xlabel = _parse_latex(xlabel, replace)
+        self.ylabel = _parse_latex(ylabel, replace)
         if hists:
             for hist in hists:
-                self.add(hist)
+                self.add(hist, replace)
     def __getitem__(self, index):
         return self.hists[index]
-    def show_titles(self, replace=None):
-        self.hists[-1].show_titles(replace=replace)
-    def add(self, hist, **kwargs):
+    def show_titles(self):
+        self.hists[-1].show_titles()
+    def add(self, hist, replace=None, **kwargs):
         """
         Add a Hist object to this stack.
 
@@ -209,43 +219,78 @@ class HistStack(object):
         when the stack is plotted.
         """
         if "label" in kwargs:
-            hist.label = kwargs['label']
+            hist.label = _parse_latex(kwargs['label'], replace)
             del kwargs['label']
         self.hists.append(hist)
         self.kwargs.append(kwargs)
-    def hist(self, replace=None, **kwargs):
+    def hist(self, **kwargs):
         """
-        Make a matplotlib hist plot, with all Hists in the stack overlaid.
+        Make a matplotlib hist plot.
 
-        Any additional keyword arguments will be passed to pyplot.hist.  You
-        will probably want to include an transparency value (i.e. alpha=0.5).
+        Any additional keyword arguments will be passed to pyplot.hist, which
+        allows a vast array of possibilities.  Particlularly, the histtype
+        values such as 'barstacked' and 'stepfilled' give completely different
+        results.  You will probably want to include a transparency value
+        (i.e. alpha=0.5).
         """
         _load_pyplot()
         contents = np.dstack([hist.y for hist in self.hists])
         xedges = self.hists[0].xedges
         x = np.dstack([hist.x for hist in self.hists])
-        plot = plt.hist(x, weights=contents, bins=xedges, **kwargs)
-    def barstack(self, replace=None, **kwargs):
+        labels = [hist.label for hist in self.hists]
+        try:
+            clist = [item['color'] for item in self.kwargs]
+            plt.gca().set_color_cycle(clist)
+            ## kwargs['color'] = clist # For newer version of matplotlib
+        except: pass
+        plot = plt.hist(x, weights=contents, bins=xedges,
+                        label=labels, **kwargs)
+    def bar3d(self, **kwargs):
+        """
+        Info
+        """
+        _load_pyplot()
+        from mpl_toolkits.mplot3d import Axes3D
+        fig = plt.figure()
+        ax = Axes3D(fig)
+        plots = []
+        labels = []
+        for i, hist in enumerate(self.hists):
+            if self.title  is not None: hist.title  = self.title
+            if self.xlabel is not None: hist.xlabel = self.xlabel
+            if self.ylabel is not None: hist.ylabel = self.ylabel
+            labels.append(hist.label)
+            all_kwargs = copy.copy(kwargs)
+            all_kwargs.update(self.kwargs[i])
+            all_kwargs.pop('fmt', None)
+            bar = ax.bar(hist.x, hist.y, zs=i, zdir='y', width=hist.width,
+                         **all_kwargs)
+            plots.append(bar)
+        #ax.yaxis.set_ticks(np.arange(len(labels)))
+        ax.yaxis.set_ticklabels(labels)
+        ax.set_ylim3d(-1, len(labels))
+        return plots
+    def barstack(self, **kwargs):
         """
         Make a matplotlib bar plot, with each Hist stacked upon the last.
 
         Any additional keyword arguments will be passed to pyplot.bar.
         """
-        bottom = [0 for i in range(self.hists[0].nbins)]
+        bottom = None # if this is set to zeroes, it fails for log y
         plots = []
         for i, hist in enumerate(self.hists):
             if self.title  is not None: hist.title  = self.title
             if self.xlabel is not None: hist.xlabel = self.xlabel
             if self.ylabel is not None: hist.ylabel = self.ylabel
-            hist.label = _parse_latex(hist.label, replace)
             all_kwargs = copy.copy(kwargs)
             all_kwargs.update(self.kwargs[i])
             all_kwargs.pop('fmt', None)
-            bar = hist.bar(replace=replace, bottom=bottom, **all_kwargs)
+            bar = hist.bar(bottom=bottom, **all_kwargs)
             plots.append(bar)
+            if not bottom: bottom = [0. for i in range(self.hists[0].nbins)]
             bottom = [sum(pair) for pair in zip(bottom, hist.y)]
         return plots
-    def errorbar(self, replace=None, **kwargs):
+    def errorbar(self, **kwargs):
         """
         Make a matplotlib errorbar plot, with all Hists in the stack overlaid.
 
@@ -256,39 +301,42 @@ class HistStack(object):
             if self.title  is not None: hist.title  = self.title
             if self.xlabel is not None: hist.xlabel = self.xlabel
             if self.ylabel is not None: hist.ylabel = self.ylabel
-            hist.label = _parse_latex(hist.label, replace)
             all_kwargs = copy.copy(kwargs)
             all_kwargs.update(self.kwargs[i])
-            errorbar = hist.errorbar(replace=replace, **all_kwargs)
+            errorbar = hist.errorbar(**all_kwargs)
             plots.append(errorbar)
         return plots
-    def bar(self, replace=None, **kwargs):
+    def bar(self, **kwargs):
         """
         Make a matplotlib bar plot, with all Hists in the stack overlaid.
 
         Any additional keyword arguments will be passed to pyplot.bar.  You will
-        probably want to include an transparency value (i.e. alpha=0.5).
+        probably want to include a transparency value (i.e. alpha=0.5).
         """
         plots = []
         for i, hist in enumerate(self.hists):
             if self.title  is not None: hist.title  = self.title
             if self.xlabel is not None: hist.xlabel = self.xlabel
             if self.ylabel is not None: hist.ylabel = self.ylabel
-            hist.label = _parse_latex(hist.label, replace)
             all_kwargs = copy.copy(kwargs)
             all_kwargs.update(self.kwargs[i])
             all_kwargs.pop('fmt', None)
-            bar = hist.bar(replace=replace, **all_kwargs)
+            bar = hist.bar(**all_kwargs)
             plots.append(bar)
         return plots
+
+    
+################ Define functions and classes for navigating within ROOT
+
 class RootFile:
     """A wrapper for TFiles, allowing easier access to methods."""
-    def __init__(self, file_name):
-        self.name = file_name[0:-5]
+    def __init__(self, file_name, name=None):
+        self.name = name
+        if name == None:
+            self.name = file_name[:-5]
         self.file = ROOT.TFile(file_name, 'read')
         if self.file.IsZombie():
-            print "Error opening %s" % file_name
-            return None
+            raise ValueError("Error opening %s" % file_name)
     def cd(self, directory=''):
         """Make directory the current directory."""
         self.file.cd(directory)
@@ -296,9 +344,6 @@ class RootFile:
         """Return a Hist object from the given path within this file."""
         return Hist(self.file.Get(object_name))
 
-
-################ Define functions for navigating within ROOT
-
 def ls(directory=""):
     """Return a python list of ROOT object names from the given directory."""
     keys = ROOT.gDirectory.GetDirectory(directory).GetListOfKeys()
@@ -331,7 +376,6 @@ def _parse_latex(string, replacements):
         return string
     for key, value in replacements:
         string = string.replace(key, value)
-    # print "LaTeX string: ", string
     if re.match(_all_whitespace_string, string):
         return ""
     return string
@@ -423,10 +467,6 @@ def axes_broken_y(axes, upper_frac=0.5, break_frac=0.02, ybounds=None,
     # Make original axes invisible
     axes.set_xticks([])
     axes.set_yticks([])
-    print upper_axes.yaxis.get_label().get_position()
-    print lower_axes.yaxis.get_label().get_position()
-    print axes.yaxis.get_label().get_position()
-    print axes.yaxis.labelpad
     for loc, spine in axes.spines.iteritems():
         spine.set_color('none')
     return upper_axes, lower_axes
@@ -455,3 +495,8 @@ def prepare_efficiency(axes, lower_bound=0.69):
 ## ##                 [plt.ylim()[j] - ywidth, plt.ylim()[j] + ywidth],
 ## ##                 linewidth=1, color='black', clip_on=False)
 ## ##             axes.add_line(line)
+
+
+if __name__ == "__main__":
+    import doctest
+    doctest.testmod()
