@@ -4,7 +4,7 @@ Draw histograms to image files, specifying options on the command line.
 
 You can overlay plots from multiple ROOT files with identical structure.
 """
-__version__ = '1.3'
+__version__ = '1.4'
 
 ## Created by Jeff Klukas (klukas@wisc.edu), November 2009
 ## Updated March 2010
@@ -222,24 +222,26 @@ def process_hist_matplotlib(path, new_path, files, obj):
     stack = r2m.HistStack()
     #### Apply options to hist from each file
     for i, file in enumerate(files):
-        hist = r2m.Hist(file.file.GetDirectory(path).Get(name), 
-                        xlabel=options.xlabel, ylabel=options.ylabel)
-        if not hist: continue
+        title, xlabel, ylabel = '', '', ''
         if options.title:
             if options.title[0] == "+":
-                hist.title += options.title[1:]
+                title += options.title[1:]
             else:
-                hist.title = options.title
+                title = options.title
         if options.xlabel:
             if options.xlabel[0] == "+":
-                hist.xlabel += options.xlabel[1:]
+                xlabel += options.xlabel[1:]
             else:
-                hist.xlabel = options.xlabel
+                xlabel = options.xlabel
         if options.ylabel:
             if options.ylabel[0] == "+":
-                hist.ylabel += options.ylabel[1:]
+                ylabel += options.ylabel[1:]
             else:
-                hist.ylabel = options.ylabel
+                ylabel = options.ylabel
+        hist = r2m.Hist(file.file.GetDirectory(path).Get(name),
+                        replace=options.replace,
+                        xlabel=xlabel, ylabel=ylabel)
+        if not hist: continue
         if options.efficiency_from:
             denom = r2m.Hist(file.file.Get(options.efficiency_from))
             eff, up, down = wilson([sum(hist.y)], [sum(denom.y)])
@@ -253,16 +255,13 @@ def process_hist_matplotlib(path, new_path, files, obj):
         else:               fmt = 'o'
         stack.add(hist, label=file.name, color=options.colors[i], fmt=fmt)
     if (options.errorbar):
-        plots = stack.errorbar(replace=options.replace, yerr=True)
+        plots = stack.errorbar(yerr=True)
     if options.stack:
-        stack.barstack(replace=options.replace)
+        stack.barstack()
     if options.bar:
-        stack.bar(replace=options.replace, alpha=0.5)
+        stack.bar(alpha=0.5)
     if options.hist:
-        mycolors = [(r,g,b,1.) for r,g,b in options.colors[:len(files)]]
-        stack.hist(replace=options.replace, alpha=0.5,
-                   # TODO: figure out how to pass a sequence to 'color'
-                   histtype='stepfilled')
+        stack.hist(alpha=0.5, histtype='stepfilled')
     if options.efficiency:
         plt.ylim(0, 1)
     if options.logx: ax.set_xscale('log')
@@ -270,7 +269,7 @@ def process_hist_matplotlib(path, new_path, files, obj):
     if options.numbering:
         plt.text(0.98, 0.98, counter, transform=plt.figure(1).transFigure,
                  size="small", ha='right', va='top')
-    stack.show_titles(replace=options.replace)
+    stack.show_titles()
     if options.legend != 'None':
         plt.legend(numpoints=1, loc=options.legend)
     output_file_name = '%s/%s/%s' % (options.plot_dir, path, name)
@@ -521,7 +520,7 @@ Note: By default, this script will work completely in ROOT.  If you use the
     parser.add_option('--bar', action="store_true", default=False,
                       help="output a matplotlib bar graph")
     parser.add_option('--hist', action="store_true", default=False,
-                      help="output a matplotlib hist graph")
+                      help="output a matplotlib hist graph (with solid fill")
     group3.add_option('--transparent', action="store_true", default=False,
                       help="use a transparent background for matplotlib "
                       "output")
